#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 1A.data - DataFrame et Matrice (correction)
# 
# Correction.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Exercice 1 : créer un fichier Excel
#     
# On souhaite récupérer les données [donnees_enquete_2003_television.txt](http://www.xavierdupre.fr/enseignement/complements/donnees_enquete_2003_television.txt) (source : [INSEE](http://www.insee.fr/fr/themes/detail.asp?ref_id=fd-hdv03&page=fichiers_detail/HDV03/telechargement.htm)).
# 
# * ``POIDSLOG`` : Pondération individuelle relative
# * ``POIDSF`` : Variable de pondération individuelle
# * ``cLT1FREQ`` : Nombre d'heures en moyenne passées à regarder la télévision 
# * ``cLT2FREQ`` : Unité de temps utilisée pour compter le nombre d'heures passées à regarder la télévision, cette unité est représentée par les quatre valeurs suivantes
#     * 0 : non concerné
#     * 1 : jour
#     * 2 : semaine
#     * 3 : mois 
#     
# Ensuite, on veut :
# 
# 1. Supprimer les colonnes vides
# 2. Obtenir les valeurs distinctes  pour la colonne ``cLT2FREQ``
# 3. Modifier la matrice pour enlever les lignes pour lesquelles l'unité de temps (cLT2FREQ) n'est pas renseignée ou égale à zéro.
# 4. Sauver le résultat au format Excel.
# 
# Vous aurez peut-être besoin des fonctions suivantes :
# 
# * [numpy.isnan](http://docs.scipy.org/doc/numpy/reference/generated/numpy.isnan.html)
# * [DataFrame.apply](http://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.apply.html)
# * [DataFrame.fillna](http://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.fillna.html) ou 
# [DataFrame.isnull](http://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.isnull.html)
# * [DataFrame.copy](http://pandas.pydata.org/pandas-docs/dev/generated/pandas.DataFrame.copy.html)

# In[2]:


from pyquickhelper.loghelper import get_url_content
import pandas, io
from ensae_teaching_cs.data import donnees_enquete_2003_television
text = donnees_enquete_2003_television()
df = pandas.read_csv(text, sep="\t")
df.head()


# In[3]:


df = df [["POIDLOG","POIDSF","cLT1FREQ","cLT2FREQ"]]  # question 1
fill = df.copy()
fill.cLT2FREQ.fillna(0, inplace=True)
distinct = set(fill["cLT2FREQ"])  # question 2
print(distinct)


# In[4]:


notnull = df [ ~df.cLT2FREQ.isnull() ]  # question 3
print(len(df),len(notnull))
notnull.tail()


# In[5]:


notnull.to_excel("data.xlsx")  # question 4


# Pour lancer Excel, vous pouvez juste écrire ceci :

# In[6]:


import os
os.system("data.xlsx")  # pour lancer Excel


# Et vous devriez voir ce qui suit :

# In[7]:


from pyquickhelper.helpgen import NbImage
NbImage("td10exc.png")


# ## Exercice 2 : moyennes par groupes
#     
# Toujours avec le même jeu de données ([marathon.txt](http://www.xavierdupre.fr/enseignement/complements/marathon.txt)), on veut ajouter une ligne à la fin du tableau croisé dynamique contenant la moyenne en secondes des temps des marathons pour chaque ville.

# In[8]:


from ensae_teaching_cs.data import marathon
import pandas
df = pandas.read_csv(marathon(filename=True), 
                     sep="\t", names=["ville", "annee", "temps","secondes"])
df.head()


# La solution requiert trois étapes.
# 
# 1. Pour avoir la moyenne par villes, il faut grouper les lignes associées à la même villes.
# 2. Ensuite, il faut introduire ces moyennes dans la table initiale : on fusionne.
# 3. On effectue le même pivot que dans l'énoncé.

# In[9]:


# étape 1
# par défaut, la méthode groupby utilise la clé de group comme index
# pour ne pas le faire, il faut préciser as_index = False
gr = df[["ville","secondes"]].groupby("ville", as_index=False).mean()
gr.head()


# In[10]:


# étape 2 - on ajoute une colonne
tout = df.merge( gr, on="ville")
tout.head()


# In[11]:


# étape 3
piv = tout.pivot("annee","ville","secondes_x")
piv.tail()


# A partir de là, on ne voit pas trop comment s'en sortir. Voici ce que je propose :
# 
# 1. On effectue un pivot sur la petite matrice des moyennes.
# 2. On ajoute ce second pivot avec le premier (celui de l'énoncé).

# In[12]:


gr["annee"] = "moyenne"
pivmean = gr.pivot("annee","ville","secondes")
pivmean


# In[13]:


piv = df.pivot("annee","ville","secondes")
pandas.concat( [ piv, pivmean ]).tail()


# En résumé, cela donne (j'ajoute aussi le nombre de marathons courus) :

# In[14]:


from ensae_teaching_cs.data import marathon
import pandas
df = pandas.read_csv(marathon(filename=True), 
                     sep="\t", names=["ville", "annee", "temps","secondes"])
gr = df[["ville","secondes"]].groupby("ville", as_index=False).mean()
gr["annee"] = "moyenne"
pivmean = gr.pivot("annee","ville","secondes")
piv = df.pivot("annee","ville","secondes")

# bizarrement, la méthode count réintroduit l'index que je pensais avoir supprimé avec
# as_index=False.
# je le supprime de nouveau avec la méthode reset_index.
# la fonction count ne compte que les valeurs non manquantes.
df2 = df.copy()
df2["nb"]=1
grc = df2[["ville","secondes","nb"]].groupby("ville", as_index=False).sum()
grc["annee"] = "nb"
pivc = grc.pivot("annee","ville","nb")

res = pandas.concat( [ piv, pivmean, pivc ])
res.tail()


# ## Exercice 3 : régression linéaire
# 
# On reproduit le début du programme proposé par l'énoncé.

# In[15]:


import pandas
from datetime import datetime, time
from ensae_teaching_cs.data import marathon
df = pandas.read_csv(marathon(filename=True), 
                     sep="\t", names=["ville", "annee", "temps","secondes"])
df = df [ (df["ville"] == "BERLIN") | (df["ville"] == "PARIS") ] 
for v in ["PARIS","BERLIN"]:
    df["est" + v] = df.apply( lambda r : 1 if r["ville"] == v else 0, axis=1)
df.head(n = 3)


# On va d'abord convertir la matrice en matrice ``numpy.matrix`` puis faire les calculs avec les notations matricielles :

# In[16]:


import numpy
Y = numpy.matrix(df[["secondes"]].values)
X = numpy.matrix(df[["annee","estPARIS","estBERLIN"]].values)
A = (X.T * X).I * X.T * Y
A


# In[17]:


numpy.column_stack([Y,X*A,X[:,1]])


# In[18]: